@@ -15,13 +15,13 @@
 #include "input_event.h"
 #include "kscreenintegration.h"
 #include "outputconfiglogging.h"
+#include "utils/orientationsensor.h"
 #include "workspace.h"
 
 #include <QFile>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
-#include <QOrientationReading>
 #include <ranges>
 
 namespace KWin
@@ -37,7 +37,7 @@ OutputConfigurationStore::~OutputConfigurationStore()
     save();
 }
 
-std::optional<std::pair<OutputConfiguration, OutputConfigurationStore::ConfigType>> OutputConfigurationStore::queryConfig(const QList<BackendOutput *> &outputs, bool isLidClosed, QOrientationReading *orientation, bool isTabletMode)
+std::optional<std::pair<OutputConfiguration, OutputConfigurationStore::ConfigType>> OutputConfigurationStore::queryConfig(const QList<BackendOutput *> &outputs, bool isLidClosed, AccelerometerOrientation orientation, bool isTabletMode)
 {
     QList<BackendOutput *> relevantOutputs;
     std::copy_if(outputs.begin(), outputs.end(), std::back_inserter(relevantOutputs), [](BackendOutput *output) {
@@ -63,7 +63,7 @@ std::optional<std::pair<OutputConfiguration, OutputConfigurationStore::ConfigTyp
     return std::make_tuple(config, ConfigType::Generated);
 }
 
-void OutputConfigurationStore::applyOrientationReading(OutputConfiguration &config, const QList<BackendOutput *> &outputs, QOrientationReading *orientation, bool isTabletMode)
+void OutputConfigurationStore::applyOrientationReading(OutputConfiguration &config, const QList<BackendOutput *> &outputs, AccelerometerOrientation orientation, bool isTabletMode)
 {
     const auto output = std::find_if(outputs.begin(), outputs.end(), [&config](BackendOutput *output) {
         return output->isInternal() && config.changeSet(output)->enabled.value_or(output->isEnabled());
@@ -77,24 +77,25 @@ void OutputConfigurationStore::applyOrientationReading(OutputConfiguration &conf
         changeset->transform = changeset->manualTransform;
         return;
     }
-    const auto panelOrientation = (*output)->panelOrientation();
-    switch (orientation->orientation()) {
-    case QOrientationReading::Orientation::TopUp:
-        changeset->transform = panelOrientation;
+    // NOTE that udev "corrects" orientation sensors to match the panel orientation,
+    // the sensor values we get are already relative to the display, not the device.
+    switch (orientation) {
+    case AccelerometerOrientation::TopUp:
+        changeset->transform = OutputTransform::Kind::Normal;
         return;
-    case QOrientationReading::Orientation::TopDown:
-        changeset->transform = panelOrientation.combine(OutputTransform::Kind::Rotate180);
+    case AccelerometerOrientation::TopDown:
+        changeset->transform = OutputTransform::Kind::Rotate180;
         return;
-    case QOrientationReading::Orientation::LeftUp:
-        changeset->transform = panelOrientation.combine(OutputTransform::Kind::Rotate90);
+    case AccelerometerOrientation::LeftUp:
+        changeset->transform = OutputTransform::Kind::Rotate90;
         return;
-    case QOrientationReading::Orientation::RightUp:
-        changeset->transform = panelOrientation.combine(OutputTransform::Kind::Rotate270);
+    case AccelerometerOrientation::RightUp:
+        changeset->transform = OutputTransform::Kind::Rotate270;
         return;
-    case QOrientationReading::Orientation::FaceUp:
-    case QOrientationReading::Orientation::FaceDown:
+    case AccelerometerOrientation::FaceUp:
+    case AccelerometerOrientation::FaceDown:
         return;
-    case QOrientationReading::Orientation::Undefined:
+    case AccelerometerOrientation::Undefined:
         changeset->transform = changeset->manualTransform;
         return;
     }
@@ -668,7 +669,7 @@ OutputConfiguration OutputConfigurationStore::generateConfig(const QList<Backend
             .rgbRange = existingData.rgbRange.value_or(kscreenChangeSet.rgbRange.value_or(BackendOutput::RgbRange::Automatic)),
             .vrrPolicy = existingData.vrrPolicy.value_or(kscreenChangeSet.vrrPolicy.value_or(VrrPolicy::Never)),
             .highDynamicRange = existingData.highDynamicRange.value_or(false),
-            .referenceLuminance = existingData.referenceLuminance.value_or(std::clamp(output->maxAverageBrightness().value_or(200), 200.0, 500.0)),
+            .referenceLuminance = existingData.referenceLuminance.value_or(std::clamp(output->maxAverageBrightnessOverride().value_or(output->advertisedMaxAverageBrightness().value_or(200)), 200.0, 500.0)),
             .wideColorGamut = existingData.wideColorGamut.value_or(false),
             .autoRotationPolicy = existingData.autoRotation.value_or(BackendOutput::AutoRotationPolicy::InTabletMode),
             .colorProfileSource = existingData.colorProfileSource.value_or(BackendOutput::ColorProfileSource::sRGB),
@@ -1159,7 +1160,7 @@ void OutputConfigurationStore::load()
             }
             if (const auto it = outputData.find("outputIndex"); it != outputData.end()) {
                 const int index = it->toInt(-1);
-                if (index <= -1 || size_t(index) >= outputDatas.size()) {
+                if (index <= -1 || size_t(index) >= outputDatas.size() || !outputDatas[index].has_value()) {
                     fail = true;
                     break;
                 }
@@ -1199,7 +1200,11 @@ void OutputConfigurationStore::load()
                 state.priority = INT_MAX;
             }
             if (const auto it = outputData.find("replicationSource"); it != outputData.end()) {
-                state.replicationSource = it->toString();
+                const QString replicationSource = it->toString();
+                const OutputState &sharedState = outputDatas[state.outputIndex].value();
+                if (sharedState.uuid != replicationSource) {
+                    state.replicationSource = replicationSource;
+                }
             }
             setup.outputs.push_back(state);
         }
